;    RLE_1A_LIGHTS-1.ASM  SMART DUAL CIRCUIT RELAY MODULE
;    Copyright (C) 2012  Paul M Bealing

;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.

;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.

;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

; RLE_1A Relay Light Controller - V0.01

; PMB ELECTRONICS
; PAUL BEALING
; www2.pmb.co.nz/blog
; paul@pmb.co.nz

; Tools: MPLAB-X on Kubuntu

;******************************************************************************
;	GENERAL INTRODUCTION

; This is a general purpose 2-channel lighting control program for the
;  RLE_1A* series "smart relays module".

; This file contains the bulk of the program, only needing the p16f676.inc
;  processor definition file.
; This file was created using MPLAB-X on Kubuntu Linux but should assemble on
;  a Windows installed copy of MPLAB.

;******************************************************************************
;	Lighting controller operation

; The time delays on the 2 relays are configurable and saved in eeprom
; To enter programming mode, activate inputs 3,4,5 and apply power. Release
;  the inputs and the program continues into programming.
; The input now counts presses or momentary activations.
;  1 = relay A, 2 = relay B, time in 5 minute increments to 255 minutes (51).

; Light level is saved from the current sensor reading.
;  Select programming function 3 and save it. If a count entry is made, the
;  light level is reset to FF which disables the function.

; The switch configuration selects the feature set / operation. ?????

; 4 data bytes are stored in eeprom:
;	relay-A timeout
;	relay-B timeout
;	light level for autolighting
;	switch configuration
; switch configuration byte:
;	00000000
;	||||||||
;	|||||||\--	relay-A auto-light enable		1 = yes
;	||||||\---						1 = yes
;	|||||\----						1 = yes
;	||||\-----						1 = yes
;	|||\------						1 = yes
;	||\-------						1 = yes
;	|\--------						1 = yes
;	\---------						1 = yes



;******************************************************************************
; NOTES

; Background support routines:

;  Core Application Loop
;   This is a loop that runs every 10mS.
;   The main application code loop goes here. It cannot take longer than 10ms
;    minus the time everything else takes; say 8mS per pass.
;   Simple Input-Output and Logic is easily handled. Timing and user interface
;    can be done RTC timers and by counting passes through the Core Loop.

;  Analog:
;   Input 2 is filtered into variable ADCRES1
;    Typically used for light or temperature sensing
;   Output PUD_IP2 drives a 10K resistor that operates as a pull-up or pull-down
;    on Input-2. Define as input to float the resistor. Pull-up to allow a
;    thermistor or CDS photocell connection Input-2 to GND.

;  Input Debounce:
;    This cleans up the inputs to improve reliability and simplify application
;     programming. Inputs can connect switches or monitor other devices.
;     Therefore: terminology "pressed" = input activated
;   INPUT1 is the real-time debounced input state
;   Short-press is latched into SWACT
;   Quick double-press is latched into SWDBL
;   Long-press is latched into SWLONG
;  Input notes:
;   1. variables INPUT1, SWACT, SWDBL, SWLONG = inputs 0,0,6,5,4,3,2,1
;   2. SWACT, SWDBL and SWLONG bits must be cleared after action
;   3. use INPUT1 to monitor the current state of an input
;   4. SWACT records that an input has been momentarily activated
;   5. SWDBL records that an input has been double-pressed

; Real Time:
;    Ideal for long time delays and periodic code execution.
;    The RTC routine uses the Timer-0 interrupt to count HH:MM:SS and provide
;     periodic execution at 250mS, 500mS, 1-sec, 1-min, 1-hr and 24-hr intervals
;    Place periodic code under the appropriate section. See RTC routine
;    Timing is only as accurate as the PIC internal clock generation etc.

;  MOSFET output
;    Flags2,3 set high will turn the output on.
;    Flags2,4 set high will dim the output with a 10mS/10mS (50%) PWM.
;     handy for driving backlight LEDs.

;  LED on-board:
;    The LED is connected to +5V, inverting the drive output; 1 = OFF.
;     flags2,0 = flash LED slow (50/50, 500mS-on/off)
;     flags2,1 = flash LED fast (50/50, 250mS-on/off)
;     flags2,2 = strobe LED (10mS pulse every 1-sec)
;    Set one of the flag bits to enable that flash.
;     Priority = 1st to last = fast then slow then strobe

;  Scope Debug output:
;    Useful during program development to monitor variables in real-time.
;    This is a serial output used to send debug bytes to a scope.
;    Update rate is 10mS. 1 byte takes about 100uS to output.
;    The number of bytes should be limited to 3 or 4 to minimise the effect
;     other routines.
;    See the Interrupt service routine to set output variables.

;  EEPROM read/write
;    At startup 4 bytes are read from the internal EEPROM. 1 "test byte" and
;     3 bytes of data. The test byte indicates that the eeprom has already
;     been programmed with default values.
;    If the test byte does not match, it is assumed that the EEPROM is not
;     programmed and it is loaded with a default configuration.
;    The test byte is the first byte at address h'00'.
;    Subroutines allow reading and writing EEPROM byte by byte.

;******************************************************************************
;	Version Notes



;******************************************************************************
;		O T H E R    N O T E S
;******************************************************************************

;************
; Reading a CDS light sensor on Input-2:


;************
; Reading a thermistor on Input-2:

; This is good enough for room temperature control and general heating and
;  cooling applications but it's probably not precise enough for some uses. It
;  has been used very sucessfully to control a fridge compressor.
; You can connect the thermistor via an op-amp and "adjust" the range etc. for
;  higher precision but that's probably better done with another board and a
;  higher level language.
; The general idea is to not worry about  the precise temperature as converting
;  the voltage from the thermistor divider to temperature requires a fair bit
;  of processing. Basic thermostat type control can be done with simple window
;  camparisons.

; ADC readings using a 10K thermistor Input-2 to GND.
;  NOTE: check these figures with your configuration

; 10001001	137	22.0
; 10001010	138
; 10001011	139	22.3
; 10001100	140
; 10001101	141
; 10001110	142
; 10001111	143	19.8
; ??
; 10110000	176	 8.7
; 10110001	177	 8.1
; ??
; 10110110	182	 6.8  defrost on temp
; 10110111	183	 6.0
; 10111000	184	 5.2  turn on compressor - 3rd try, defrost off temp
; 10111001	185	 4.6
; 10111010	186	 4.5  turn on compressor - 2nd try
; 10111011	187	 4.3
; 10111100	188	 3.3  turn on compressor - 1st try,   turn off compressor - 3rd try
; 10111101	189	 2.7
; 10111110	190	 2.2  turn off compressor - 2nd try
; 10111111	191	 1.9
; 11000000	192	 1.6
; 11000001	193	 1.3
; 11000010	194	 1.0  turn off compressor - 1st try
; 11000011	195	 0.0

;************
; Thermistor sensor interfacing

; For fridge/freezer control using a simple voltage divider into the ADC does not provide
;  much range (185-195 decimal = 4-0 dec-C).
; For better resolution the voltage divider output should be scaled and offset.


;******************************************************************************
; DEFAULT IO CONFIG FOR RLE_1A4 BOARD
;******************************************************************************

; DAT		EQU	h'0'		; PORT-A input	ICSP data
; EXT_IO		EQU	h'0'		; PORT A output	diagnostic
; RESET_IP	EQU	h'0'		; PORT A input	dual reset inputs
; CLK		EQU	h'1'		; PORT-A input	ICSP clock
; INPUT_2		EQU	h'1'		; PORT-A input	input 2
; INPUT_3		EQU	h'2'		; PORT-A input	input 3
; ;		EQU	h'3'		; PORT-A input	MCLR
; IP2_OP		EQU	h'4'		; PORT-A input	input 2 level driver
; LED_OP		EQU	h'5'		; PORT-A output	LED output
;
; INPUT_4		EQU	h'0'		; PORT-C input	input 4
; INPUT_5		EQU	h'1'		; PORT-C input	input 5
; INPUT_6		EQU	h'2'		; PORT-C input	input 6
; RELAY_A		EQU	h'3'		; PORT-C output	relay A
; RELAY_B		EQU	h'4'		; PORT-C output	relay B
; OUTPUT		EQU	h'5'		; PORT-C output	mosfet output
;
; DDRA_0		EQU	b'11001111'	; PORT-A data direction default
; DDRC_0		EQU	b'11000111'	; PORT-C data direction default

;******************************************************************************
; FLAGS USED BY PROGRAM
;******************************************************************************

;	FLAGS1
;	00000000
;	||||||||
;	|||||||\--	10mS Core-Loop flag 			1 = yes
;	||||||\---	Programming mode enabled		1 = yes
;	|||||\----						1 = yes
;	||||\-----						1 = yes
;	|||\------	relay_A auto-light, done		1 = yes
;	||\-------						1 = yes
;	|\--------	ADC pass # (00 = #1, 01 = #2)		1 = pass 0-1
;	\---------	1-second periodic flag			1 = yes

;  FLAGS2
;     00000000
;	||||||||
;	|||||||`-	flash LED slow				; 1 = yes
;	||||||`--	flash LED fast				; 1 = yes
;	|||||`---	pulse LED (10mS)			; 1 = yes
;	||||`----	output on				; 1 = yes
;	|||`-----	output half/full			; 1 = full
;	||`------	startup/reset (1st pass)		; 1 = yes
;	|`-------	relay_A timer, running			; 1 = yes
;	`--------	relay_B timer, running			; 1 = done

;  FLAGS3
;	00000000
;	||||||||
;	|||||||\--	timer (1) running			1 = yes
;	||||||\---	timer (1) timed out			1 = yes
;	|||||\----	timer (2) running			1 = yes
;	||||\-----	timer (2) timed out			1 = yes
;	|||\------	timer (3) running			1 = yes
;	||\-------						1 = yes
;	|\--------	timer (4) running			1 = yes
;	\---------	timer (4) timed out			1 = yes

;	FLAGS4
;	00000000
;	||||||||
;	|||||||\--						1 = yes
;	||||||\---						1 = yes
;	|||||\----						1 = yes
;	||||\-----						1 = yes
;	|||\------						1 = yes
;	||\-------						1 = yes
;	|\--------						1 = yes
;	\---------						1 = yes

;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************

;******************************************************************************
;******************************************************************************
;*                         D E C L A R A T I O N S                            *
;******************************************************************************
;******************************************************************************

	list		p=16F676		; define processor
	#include	<p16f676.inc>		; processor definition header file

;==========================================================================
;       Configuration Bits
;==========================================================================

;	__CONFIG	_CP & _CPD_OFF & _BODEN & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG	_CP_OFF & _CPD_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT

;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;*             O P E R A T I O N A L   C O N F I G   S E T U P                *
;******************************************************************************
;******************************************************************************

	CONSTANT NORMAL=1		; normal timing
	CONSTANT SHORTER=0		; shorter timing
	CONSTANT TESTING=0		; Ttest mode (faster)

;******************************************************************************

	if	NORMAL
DON_TMP	    EQU	b'10111010'	; turn on  temperature - defrost cycle
DOF_TMP	    EQU	b'10111110'	; turn off temperature - defrost cycle
NON_TMP	    EQU	b'10111010'	; turn on  temperature - normal
NOF_TMP	    EQU	b'10111110'	; turn off temperature - normal
RS_DLYM	    EQU	  d'03'		; mins		; reset startup delay
RS_DLYS	    EQU	  d'00'		; secs		;  "
SU_DELM	    EQU	d'05'		; mins		; start-up delay for door opened test
SU_DELS	    EQU	d'00'		; secs		;  "
MIN_ONM	    EQU	  d'02'		; mins		; minimum run time once ON
MIN_ONS	    EQU	  d'00'		; secs		;  "
MAX_ONM	    EQU	d'20'		; mins		; maximum run time per cycle
MAX_ONS	    EQU	d'00'		; secs		;  "
MIN_OFM	    EQU	  d'05'		; mins		; minimum off time per cycle
MIN_OFS	    EQU	  d'00'		; secs		;  "
DEF_CYC	    EQU	d'24'		; hours		; run time between defrost cycles
DEF_TIM	    EQU	d'03'		; hours		; run time of defrost cycle
	endif

	if	SHORTER
DON_TMP	    EQU	b'10110110'	; turn on  temperature - defrost cycle
DOF_TMP	    EQU	b'10111000'	; turn off temperature - defrost cycle
NON_TMP	    EQU	b'10111000'	; turn on  temperature - normal
NOF_TMP	    EQU	b'10111100'	; turn off temperature - normal
RS_DLYM	    EQU	d'00'		; mins		; reset startup delay
RS_DLYS	    EQU	d'15'		; secs		;  "
SU_DELM	    EQU	d'01'		; mins		; start-up delay for door opened test
SU_DELS	    EQU	d'01'		; secs		;  "
MIN_ONM	    EQU	d'02'		; mins		; minimum run time once ON
MIN_ONS	    EQU	d'01'		; secs		;  "
MAX_ONM	    EQU	d'06'		; mins		; maximum run time per cycle
MAX_ONS	    EQU	d'01'		; secs		;  "
MIN_OFM	    EQU	d'02'		; mins		; minimum off time per cycle
MIN_OFS	    EQU	d'01'		; secs		;  "
DEF_CYC	    EQU	d'03'		; hours		; run time between defrost cycles
DEF_TIM	    EQU	d'01'		; hours		; run time of defrost cycle
	endif

	if	TESTING
DON_TMP	    EQU	b'10110110'	; turn on  temperature - defrost cycle
DOF_TMP	    EQU	b'10111000'	; turn off temperature - defrost cycle
NON_TMP	    EQU	b'10111000'	; turn on  temperature - normal
NOF_TMP	    EQU	b'10111100'	; turn off temperature - normal
RS_DLYM	    EQU	d'00'		; mins		; reset startup delay
RS_DLYS	    EQU	d'05'		; secs		;  "
SU_DELM	    EQU	d'01'		; mins		; start-up delay for door opened test
SU_DELS	    EQU	d'05'		; secs		;  "

MIN_ONM	    EQU	d'00'		; mins		; minimum run time once ON
MIN_ONS	    EQU	d'30'		; secs		;  "
MAX_ONM	    EQU	d'02'		; mins		; maximum run time per cycle
MAX_ONS	    EQU	d'00'		; secs		;  "
MIN_OFM	    EQU	d'01'		; mins		; minimum off time per cycle
MIN_OFS	    EQU	d'00'		; secs		;  "
DEF_CYC	    EQU	d'03'		; hours		; run time between defrost cycles
DEF_TIM	    EQU	d'01'		; hours		; run time of defrost cycle
	endif

;******************************************************************************
; RAM VARIABLE ASSIGNMENTS

RAM_0	UDATA_SHR	h'0020'	        	; USER RAM ADDRESS

FLAGS1	res	1	; 20		; GP FLAGS REGISTER #1
FLAGS2	res	1	; 21		; GP FLAGS REGISTER #2
FLAGS3	res	1	; 22		; GP FLAGS REGISTER #3
FLAGS4	res	1	; 23		; GP FLAGS REGISTER #4
SAV_W1	res	1	; 24		; place to save W
SAV_ST1	res	1	; 25		; place to save STATUS
SAV_FSR	res	1	; 26		; place to save FSR
SAV_PC1	res	1	; 27		; place to save PCLATH
TIC	res	1	; 28		; Number of 10ms intervals/sec 0-99.
SEC	res	1	; 29		; Number of seconds 0-59.
MIN	res	1	; 2A		; Number of minutes.
HRS	res	1	; 2B		; Number of hours.
					;
GPTIM1	res	1	; 2C		; general purpose timer (INTERRUPT BASED)
GPTIM2	res	1	; 2D		; general purpose timer
GPTIM3	res	1	; 2E		; general purpose timer (LOOP BASED SHORT DELAYS)
CNTR1	res	1	; 2F		; config mode timer
CNTR2	res	1	; 30		; ?
ADCRES1	res	1	; 31		; LAST ADC RESULT (pot)
					;
SWONT	res	1	; 32		; switch timer
SWOFT	res	1	; 33		; switch timer
SWTIM	res	1	; 34		; switch timer
SWTMP	res	1	; 35		; switch temporary variable
SWTP	res	1	; 36		; switch variable, this pass
SWLP1	res	1	; 37		; switch variable, last pass
SWLP2	res	1	; 38		; switch variable, last pass -1
SWACT1	res	1	; 39		; activation history -1
SWACT2	res	1	; 3A		; activation history -2
SWSTAT	res	1	; 3B		; switch variable, processing status
SWACT	res	1	; 3C		; switch variable, ACTIVATED
SWDBL	res	1	; 3D		; switch variable, DOUBLE PRESSED
SWLONG	res	1	; 3E		; switch variable, LONG PRESSED
INPUT1	res	1	; 3F		; input status register
					;
CNT_RA	res	1	; 40		; delay for Relay_A
CNT_RB	res	1	; 41		; delay for Relay_B
CNT_RBA	res	1	; 42		; delay for Relay_A (auto-light)
					;
TMP1	res	1	; 43		; temp value #1 (DAT_TX)
TMP2	res	1	; 44		; temp value #2 (DAT_TX)
TMP3	res	1	; 45		; temp value #3 (DAT_TX)
					;
EE_DAT	res	1	; 46		; temp variable	for internal eeprom access
EE_ADD	res	1	; 47		; temp variable	for internal eeprom access
					;
PAUL	res	1	; 48		; debug variable
					;
CFG_PP	res	1	; 49		; 00 from eeprom = programmed ?
CFG_RA	res	1	; 4A		; 01 from eeprom = relay A timeout
CFG_RB	res	1	; 4B		; 02 from eeprom = relay B timeout
CFG_LL	res	1	; 4C		; 03 from eeprom = light level
CFG_SC	res	1	; 4D		; 04 from eeprom = switch configuration

PM_CNT1	res	1	; 4E		; programming mode variable
PM_CNT2	res	1	; 4F		; programming mode variable
PM_CNT3	res	1	; 50		; programming mode variable
PM_CNT4	res	1	; 51		; programming mode variable

			; end = 5F (64 bytes of RAM)

;******************************************************************************
;******************************************************************************
; CONSTANTS
;******************************************************************************
;******************************************************************************

; for INPUT1, SWACT, SWDBL, SWLONG (for bsf,bcf etc..)

IP_1		EQU	h'0'		; input-1	1 & 2	reset
IP_2		EQU	h'1'		; input-2	3	analog
IP_3		EQU	h'2'		; input-3	4	relay-A
IP_4		EQU	h'3'		; input-4	5	relay-A
IP_5		EQU	h'4'		; input-5	6	relay-B
IP_6		EQU	h'5'		; input-6	7	relay-B
SW_R		EQU	h'0'		; switch-reset	1 & 2	reset
SW_1		EQU	h'2'		; switch-1	4	relay-A
SW_3		EQU	h'3'		; switch-3	5	relay-A
SW_2		EQU	h'4'		; switch-2	6	relay-B
SW_4		EQU	h'5'		; switch-4	7	relay-B

; hardware I/O

DAT		EQU	h'0'		; PORT-A input	ICSP data
EXT_IO		EQU	h'0'		; PORT A output	diagnostic
RESET_IP	EQU	h'0'		; PORT A input	dual reset inputs
CLK		EQU	h'1'		; PORT-A input	ICSP clock
INPUT_2		EQU	h'1'		; PORT-A input	input 2
INPUT_3		EQU	h'2'		; PORT-A input	input 3
;		EQU	h'3'		; PORT-A input	MCLR
PUD_IP2		EQU	h'4'		; PORT-A output	input 2 level driver
LED_OP		EQU	h'5'		; PORT-A output	LED output

INPUT_4		EQU	h'0'		; PORT-C input	input 4
INPUT_5		EQU	h'1'		; PORT-C input	input 5
INPUT_6		EQU	h'2'		; PORT-C input	input 6
RELAY_A		EQU	h'3'		; PORT-C output	relay A
RELAY_B		EQU	h'4'		; PORT-C output	relay B
OUTPUT		EQU	h'5'		; PORT-C output	mosfet output

DDRA_0		EQU	b'11001111'	; PORT-A data direction default
DDRC_0		EQU	b'11000111'	; PORT-C data direction default

;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;*           R E S E T   A N D   I N T E R R U P T   V E C T O R S            *
;******************************************************************************

RESET_VECTOR	CODE	h'000'		; processor reset vector
		NOP				; NOP for ICD
		GOTO	START			;

INT_VECTOR	CODE	h'004'		; processor reset vector
		GOTO	ISR			;

;************************************************************************
; copyright notice
;************************************************************************

MSG1	DT	"RLE_1A_LIGHTS-1_01"		; RLE_1A_LIGHTS-1_01
MSG2	DT	"(C) PMB 2012",0		; (C) PMB 2012

;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;		I N T E R R U P T   S E R V I C E
;******************************************************************************
;       This Handler routes interrupts to the correct ISR.
;******************************************************************************

					; save registers
ISR:	MOVWF	SAV_W1				; save "W"
	MOVF	STATUS,W			; move "STATUS" to be saved into "W"
	CLRF	STATUS				; force to bank 0 (RAM)
	MOVWF	SAV_ST1				; save status
	MOVF	PCLATH,W			; save "PCLATH"
	MOVWF	SAV_PC1				;  "
	BCF	PCLATH,3			; 1st page
	BCF	PCLATH,4			;  "

; determin interrupt source

ISR_1	BTFSC	INTCON,T0IF			; test flag?
	GOTO	ISR_TO				; timer interrupt service routine
	BTFSC	INTCON,INTF			; test flag?
	GOTO	ISR_INT				; external interrupt service routine

;************
; Common to all ISR's

					; diagnostic o/p
ISR_90	MOVF	ADCRES1,W			;
	CALL	DAT_TX				;  diagnostic TX
;	MOVF	CFG_LL,W			;
;	CALL	DAT_TX				;  diagnostic TX
;	MOVF	PAUL,W				;
;	CALL	DAT_TX				;  diagnostic TX
					; finished
					; restore registers
ISR_99	MOVF	SAV_PC1,W			; restore "PCLATH"
	MOVWF	PCLATH				;  "
	MOVF	SAV_ST1,W			; restore "STATUS" (original bank)
	MOVWF	STATUS				;  "
	MOVF	SAV_W1,W			; restore "W"
					; clean & quit
	BCF	INTCON,INTF			; clear external interrupt flag
	BSF	INTCON,INTE			; re-enable external interrupt
	RETFIE					; return, enabling global INT

;******************************************************************************
;		EXTERNAL INTERRUPT SERVICE ROUTINE
;******************************************************************************

ISR_INT:	BCF	INTCON,INTF			; clear external interrupt flag
		BCF	INTCON,INTE			; disable external interrupt
		GOTO	ISR_90				; finished

;******************************************************************************
;		TIMER OVERFLOW INTERRUPT SERVICE
;******************************************************************************

ISR_TO:	BCF	INTCON,T0IF			; clear TO Interrupt Flag
						; 10ms=10,000us
						; 10,000 /256 = 39.025
	MOVLW	d'218'				; 256-39 = 217, 39counts=10ms
	MOVWF	TMR0				; restore timer value for next interval

	BSF	FLAGS1,0			; Core-Loop flag (see CORE:)

;************
; LED 10mS pulse (on at 1-sec)

ISR_OP	BTFSC	FLAGS2,0			; \ slow override pulse ?
	GOTO	ADC1A				; |  yes, skip
	BTFSC	FLAGS2,1			; | fast override pulse ?
	GOTO	ADC1A				; /  yes, skip
	BTFSC	FLAGS2,2			; LED pulse enabled ?
	BSF	PORTA,LED_OP			; LED off now
					; backlight LED control
ISR_OP1	BTFSS	FLAGS2,3			; is output on ?
	BCF	PORTC,OUTPUT			; output off
	BTFSS	FLAGS2,3			; is output on ?
	GOTO	ISR_OP9				;  no, skip
	BTFSS	FLAGS2,4			; is it dimmed ?
	GOTO	ISR_OP2				;  no, skip
	MOVF	PORTC,W				; \ read Port-A
	XORLW	1 << OUTPUT			; |  invert output
	MOVWF	PORTC				; / write Port-A
	GOTO	ISR_OP9				;  done
ISR_OP2	BSF	PORTC,OUTPUT			; output on
ISR_OP9

;************
; SAVE ADC RESULT & INITALISE ADC FOR NEXT CONVERSION

; The adc conversion result is used to adjust the value up or down of the
;  stored working result. This effectively filters the adc reading, preventing
;  extcessive variation due to noise.
;	ADCRES1 = A1 = Input-2

ADC1A:	BTFSC	FLAGS1,6			; set = pass #2
	GOTO	ADC3				; run pass #2
					; pass #1 (AN2 = 010) (TEMPERATURE)
	MOVF	ADRESH,W			; read ADC result
	SUBWF	ADCRES1,0			; test result (F - W)
	BTFSC	STATUS,Z			; test for zero
	GOTO	ADC2				; zero = no change, finished
	BTFSS	STATUS,C			; test carry (set = +ve result)
	GOTO	ADC1				; clear, -ve result, increase
	DECF	ADCRES1,F			; decrease
	GOTO	ADC2				; done
ADC1	INCF	ADCRES1,F			; increase
ADC2	BSF	ADCON0,CHS0			; select channel 1
	BCF	ADCON0,CHS1			;   "
	MOVLW	d'50'				; input settling delay time
	MOVWF	GPTIM3				;   counter
	CALL	BDEL1				;   run delay
	BSF	ADCON0,GO_DONE			; start conversion
	BSF	FLAGS1,6			; next pass = #2
	GOTO	ADC9				; done
					; pass #2 (just re-start converion)
ADC3	BSF	ADCON0,CHS0			; select channel 1
	BCF	ADCON0,CHS1			;   "
	MOVLW	d'50'				; input settling delay time
	MOVWF	GPTIM3				;   counter
	CALL	BDEL1				;   run delay
	BSF	ADCON0,GO_DONE			; start conversion
	BCF	FLAGS1,6			; next pass = #1
ADC9

;************************************************************************
; SWITCH INPUT DEBOUNCE & COUNT

; Whenever a switch is accepted as being on, the timer is started and a flag is set
;  to indicate timing.
; If the timer reaches 1.5 seconds, the switch is registered at SWLONG.
; Whenever a switch is released, the timer is started and a flag set to indicate timing.

; TRANSLATIONS:
;	SWACT, SWDBL, SWLONG
;	00000000	i/p	typical function
;	||||||||
;	|||||||\--	1,2	both reset inputs		1 = yes
;	||||||\---	3	normally analog			1 = yes
;	|||||\----	4	toggle relay A			1 = yes
;	||||\-----	5					1 = yes
;	|||\------	6	toggle relay B			1 = yes
;	||\-------	7					1 = yes
;	|\--------						1 = yes
;	\---------						1 = yes

; Prepare raw input data from Port-A and Port-C
;   port-A = 00000111 = inputs 1,2,3
;   port-C = 00000111 = inputs 4,5,6
;    becomes 00111111 = 6,5,4,3,2,1
					; prepare raw input data
RTI80A:	MOVF	PORTC,W				; \ read Port-C
	ANDLW	b'00000111'			; |  mask unused bits
	BCF	STATUS,C			; | clear carry for rotate
	MOVWF	SWTP				; | temp save for shift
	RLF	SWTP,F				; |  shift left
	RLF	SWTP,F				; |  shift left
	RLF	SWTP,F				; |  shift left
	MOVF	PORTA,W				; | read Port-A
	ANDLW	b'00000111'			; |  mask unused bits
	IORWF	SWTP,F				; / now contains true input states
					; process
	MOVF	INPUT1,W			; current state, 1's = on
	MOVWF	SWTMP				; save last pass state for activation processing
	MOVF	SWTP,W				; get "this pass"
	ANDWF	SWLP1,W				; last pass
	ANDWF	SWLP2,W				; last pass -1
	IORWF	INPUT1,W			; add any switches that are now on (1 = debounced on)
	MOVWF	INPUT1				;  & save
	MOVF	SWTP,W				; current pass
	IORWF	SWLP1,W				; last pass
	IORWF	SWLP2,W				; last pass -1
	IORLW	b'11000000'			; prevent masking of other bits
	ANDWF	INPUT1,W			; merge any switches that are now off (0 = debounced off)
	MOVWF	INPUT1				;  & save true debounced switch states

	IORWF	SWLONG,W			; mask switches active as long (prevents activate & long at same time)
	XORLW	b'00111111'			; invert, 1's = off
	ANDLW	b'00111111'			; mask SWLONG other bits
	ANDWF	SWTMP,W				; 1's = on, only if active on last pass
	MOVWF	SWTMP				; save copy
	IORWF	SWACT,W				; merge "switches have been activated"
	MOVWF	SWACT				;  & save

; save new activations to buffer (PIC)

	MOVF	SWTMP,W				; retrieve copy
	BTFSC	STATUS,Z			;  test zero
	GOTO	RTI82				; no activations to save
	MOVF	SWACT1,W			; move back for double-
	MOVWF	SWACT2				;  press detection
	MOVF	SWTMP,W
	MOVWF	SWACT1				; save for double press

; process double press (PIC)

RTI81	BTFSS	SWSTAT,5			; off period not valid
	GOTO	RTI82				;  "
	MOVF	SWACT1,W			; read
	ANDWF	SWACT2,W			; last switch to activate
	BTFSC	STATUS,Z			;  test zero
	GOTO	RTI82				;  no double press detected
	IORWF	SWDBL,W				; merge double press
	MOVWF	SWDBL				;  & save
	MOVLW	b'00111111'			; clear double press & timer flags
	ANDWF	SWSTAT,W			;  "
	MOVWF	SWSTAT				;  "
	GOTO	RTI89

; start on period timer (PIC)

RTI82	MOVF	INPUT1,W			; \
	ANDLW	b'00111111'			;  > no switches are on
	BTFSC	STATUS,Z			;  >
	GOTO	RTI84				; /
	BTFSC	SWSTAT,6			; already timing on period
	GOTO	RTI87				;  "
	CLRF	SWONT				; sw on detected, clear timer
	BSF	SWSTAT,6			; flag timer running
	BCF	SWSTAT,4			; clear timer running flag
	GOTO	RTI85				; done
RTI84	BCF	SWSTAT,6			; clear timer running flag

; start off period timer (PIC)

RTI85	MOVF	INPUT1,W			; \
	ANDLW	b'00111111'			;  >mask top nibble
	BTFSS	STATUS,Z			;  >
	GOTO	RTI87				; / at least 1 switch active
	BTFSC	SWSTAT,7			; already timing off period
	GOTO	RTI87				;  "
	BTFSC	SWSTAT,4			; has already run for this cycle
	GOTO	RTI87				;  "
	CLRF	SWOFT				; start timer for 0.1 sec
	BSF	SWSTAT,7			; flag timer running
	BSF	SWSTAT,4			;  "
	BCF	SWSTAT,6			; clear timer running flag
	BCF	SWSTAT,5			;  "

; check off period timer (PIC)
;  These 2 time periods affect double press timing. The relationship between
;   them must be maintained; or double press will not work.

RTI87	BTFSS	SWSTAT,7			; timer not running
	GOTO	RTI88				;  "
	INCF	SWOFT,F				; decrement timer
	MOVF	SWOFT,W				; read timer
	SUBLW	d'10'				; (0.1 sec)
	BTFSS	STATUS,Z			; test zero
	GOTO	RTI87A				;   not yet
	BSF	SWSTAT,5			; set double press flag
RTI87A	MOVF	SWOFT,W				; read timer
	SUBLW	d'25'				; (0.25 sec)
	BTFSS	STATUS,Z			; test zero
	GOTO	RTI88				;   not yet
	BCF	SWSTAT,7			; clear double press & timer flags
	BCF	SWSTAT,5			;  "

; check on period timer (PIC)

RTI88	BTFSS	SWSTAT,6			; timer running
	GOTO	RTI89				;  no
	INCF	SWONT,F				; increment timer
	MOVF	SWONT,W				; read timer
	SUBLW	d'50'				; long press time (0.5 second)
	BTFSC	STATUS,C			; test zero
	GOTO	RTI89				; not yet
	MOVF	INPUT1,W			; which switch is on
	IORWF	SWLONG,W			; merge
	MOVWF	SWLONG				;  & save

	ANDWF	SWACT,W				; 1 = long switch only
	XORLW	b'00111111'			; invert
	MOVWF	SWTMP				; temp save to B
	ANDWF	SWACT,W				; off long switch in SWACT
	MOVWF	SWACT				;  & save
	MOVF	SWTMP,W				; restore from temp save
	ANDWF	SWDBL,W				; off long switch in SWDBL
	MOVWF	SWDBL				;  & save
	BCF	SWSTAT,6			; clear timer running flag

; update history (PIC)

RTI89	MOVF	SWLP1,W				; roll pass data back
	MOVWF	SWLP2				;  to pass before
	MOVF	SWTP,W				; roll pass data back
	MOVWF	SWLP1				;  to pass before

; remove dbl presses from SWACT

	MOVF	SWDBL,W				; get double press
	BTFSC	STATUS,Z			;  finish if none active
	GOTO	RTC				;   "
	XORLW	b'00111111'			; invert swdbl
	ANDWF	SWACT,W				;  merge
	MOVWF	SWACT				;  & save

;************************************************************************
; UPDATE TIMER

; every 10mS

RTC:	MOVF	TIC,W				;
	SUBLW	d'100'				; test for 100 TIC count
	BTFSC	STATUS,Z			;
	GOTO	RTC1				; ripple clock
	INCF	TIC,1				;
	DECF	GPTIM1,F			; general purpose timer
	GOTO	RTC99				;

RTC1	CLRF	TIC				; restart TIC counter from zero
	GOTO	PER_1S				; 1 second functions
PER_1SR	BSF	FLAGS1,7			; set 1-sec flag
	INCF	SEC,F				; +1 seconds
	MOVF	SEC,W				; test
	SUBLW	d'60'				; check for SEC rollover
	BTFSC	STATUS,Z			;
	GOTO	RTC2				; yes
	GOTO	RTC99				; no

RTC2	CLRF	SEC				; zero seconds
	GOTO	PER_1M				; 1 minute functions
PER_1MR	INCF	MIN,F				; +1 minutes
	MOVF	MIN,W				; test
	SUBLW	d'60'				; check for MIN rollover
	BTFSC	STATUS,Z			;
	GOTO	RTC3				; yes
	GOTO	RTC99				; no

RTC3	CLRF	MIN				; zero minutes
	GOTO	PER_1H				; 1 hr functions
PER_1HR	INCF	HRS,F				; +1 hours
	MOVF	HRS,W				; test
	SUBLW	d'24'				; check for HRS rollover (normal)
	BTFSS	STATUS,Z			;
	GOTO	RTC99				;  no
	CLRF	HRS				; reset hours to 00
	GOTO	PER_24				; 00:00 (24 hr functions, if any)
PER_24R						;

;************
; Check on 250ms and 500ms real-time tasks

RTC99	MOVF	TIC,W				; read TIC
	BTFSC	STATUS,Z			; test for zero
	GOTO	PER_500				; run 500mS & 250mS tasks
	MOVF	TIC,W				;
	SUBLW	d'25'				;
	BTFSC	STATUS,Z			; test for 25
	GOTO	PER_250				; run 250mS tasks
	MOVF	TIC,W				;
	SUBLW	d'50'				;
	BTFSC	STATUS,Z			; test for 50
	GOTO	PER_500				; run 500mS & 250mS tasks
	MOVF	TIC,W				;
	SUBLW	d'75'				;
	BTFSC	STATUS,Z			; test for 75
	GOTO	PER_250				; run 250mS tasks
	GOTO	ISR_90				; RETURN - finished RTC (not 250 or 500 mS)

;------------------------------------------------------
;	500ms Periodic Processing
;------------------------------------------------------
; flash leds slowly if flags set

PER_500	BTFSC	FLAGS2,1			; \ fast override slow ?
	GOTO	P500_10				; /  yes, skip
	BTFSS	FLAGS2,0			; enabled ?
	GOTO	P500_10				;  no, skip
	MOVF	PORTA,W				; read Port-A
	XORLW	1 << LED_OP			; invert leds
	MOVWF	PORTA				; write Port-A

P500_10	NOP					; .

;       NOTE: 500ms code should 'fall through' to 250ms code.

;------------------------------------------------------
;	250ms Periodic Processing
;------------------------------------------------------

					; flash LED-2 fast
PER_250	BTFSS	FLAGS2,1			; enabled ?
	GOTO	P250_10				;  no, skip
	MOVF	PORTA,W				; \ read Port-A
	XORLW	1 << LED_OP			; | invert leds
	MOVWF	PORTA				; / write Port-A
					; GP timer 2
P250_10	BTFSS	FLAGS3,2			; timer 2 running ?
	GOTO	P250_20				;  no, skip
	DECFSZ	CNTR2,F				; decrement
	GOTO	P250_20				;  no timeout yet
	BCF	FLAGS3,2			; clear running flag
					; done
P250_20
	GOTO	ISR_90				; RETURN - finished RTC

;------------------------------------------------------
;	1 sec period
;------------------------------------------------------

; LED 10mS pulse every 1-sec

PER_1S	BTFSC	FLAGS2,0			; \ slow override pulse ?
	GOTO	P1S_2				; |  yes, skip
	BTFSC	FLAGS2,1			; | fast override pulse ?
	GOTO	P1S_2				; /  yes, skip
	BTFSC	FLAGS2,2			; LED pulse enabled ?
	BCF	PORTA,LED_OP			; LED on now
P1S_2
	GOTO	PER_1SR				; RETURN - done

;------------------------------------------------------
;	1 min period
;------------------------------------------------------

PER_1M
					; relay timeout counters
P1M_1	BTFSS	FLAGS2,6			; timer running ?
	GOTO	P1M_5				;  no, skip
	DECF	CNT_RA,F			;
	BTFSS	STATUS,Z			; zero yet ?
	GOTO	P1M_5				;  no, skip
	BCF	PORTC,RELAY_A			; relay off
	BCF	FLAGS2,6			; stop timer

P1M_5	BTFSS	FLAGS2,7			; timer running ?
	GOTO	P1M_6				;  no, skip
	DECF	CNT_RB,F			;
	BTFSS	STATUS,Z			; zero yet ?
	GOTO	P1M_6				;  no, skip
	BCF	PORTC,RELAY_B			; relay off
	BCF	FLAGS2,7			; stop timer

P1M_6

					; done
P1M_9	GOTO	PER_1MR				; RETURN - done

;------------------------------------------------------
;	1 hr period
;------------------------------------------------------

PER_1H
P1H_10	GOTO	PER_1HR				; RETURN - done

;------------------------------------------------------
;	24 hr period
;------------------------------------------------------

PER_24
P24_10	GOTO	PER_24R				; RETURN - done

;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;		P R O G R A M   C O D E   S T A R T S   H E R E
;******************************************************************************

; INITALISATION CODE

START:	BCF	INTCON,GIE			; DISABLE GLOBAL INTERRUPTS
	BTFSC	INTCON,GIE			; ensure global interrupt disabled
	GOTO	START				; not disabled

	BSF	STATUS,RP0			; select bank 1
	CALL	h'3FF'				; \ get the osc calibration value
	MOVWF	OSCCAL				; / calibrate now
	MOVLW	DDRA_0				; Port A DDR (varies during operation)
	MOVWF	TRISA				;  "
	MOVLW	DDRC_0				; port C DDR (varies also)
	MOVWF	TRISC				;  "
	MOVLW	b'10000111'			; no pullups, prescale WDT = maximum
	MOVWF	OPTION_REG			;  "
	MOVLW	b'00010000'			; configure ADC timing
	MOVWF	ADCON1				;  disabled
	MOVLW	b'00000010'			; select analog inputs (AN1)
	MOVWF	ANSEL				;  "
	BCF	STATUS,RP0			; select bank 0
	MOVLW	b'00000111'			; disable comparator
	MOVWF	CMCON				;  "
	MOVLW	b'00001001'			; configure ADC
	MOVWF	ADCON0				;  disabled, all digital
;	MOVLW	b'00110000'			; setup Timer 1 to monitor PPM in
;	MOVWF	T1CON				;  "
	MOVLW	h'00'
	MOVWF	PORTA
	MOVWF	PORTC

;************
; CLEAR RAM

	MOVLW	h'20'				; last byte of RAM
	MOVWF	FSR				; indirect addressing register
START1	CLRF	INDF				; clear byte
	INCF	FSR,F				; next address
	MOVLW	h'60'				; last byte (+1)
	SUBWF	FSR,W				;  check
	BTFSS	STATUS,Z			;  finished
	GOTO	START1				; next byte

;************
; Interrupt enable

	MOVLW	d'217'				; 217 = 10ms timeout
	MOVWF	TMR0				; preload timer.
	BSF	INTCON,GIE			; enable global interrupts
	BSF	INTCON,T0IE			; enable timer overflow interrupt

; application setup

	MOVLW	d'200'			; 200D	; preset adc results
	MOVWF	ADCRES1				;
	BSF	PORTA,PUD_IP2			; set sensor pull-up

;************
; get delay from eeprom

					; read eeprom (test byte)
EEP_RD	MOVLW	d'00'				; \ set address to read	(programmed ?)
	CALL	DM_RD				; | read byte/s now
	MOVWF	CFG_PP				; / save
					; is eeprom already programmed ?
	SUBLW	"R"				; | check character
	BTFSC	STATUS,Z			; /  "
	GOTO	EEP_RD1				; yes, skip, already programmed
					; no, program eeprom (initiate from new)
	MOVLW	"R"				; \ data (check character)
	MOVWF	EE_DAT				; |  "
	MOVLW	d'00'				; | address
	MOVWF	EE_ADD				; |  "
	CALL	DM_WR				; / write 00 byte now
	MOVLW	d'15'				; \ data (default value)
	MOVWF	EE_DAT				; |  "
	CALL	DM_WR				; | write 01 byte now
	CALL	DM_WR				; | write 02 byte now
	MOVLW	h'FF'				; | data (default LL value)
	MOVWF	EE_DAT				; |  "
	CALL	DM_WR				; | write 03 byte now
	MOVLW	b'00000110'			; | data (default SC value)
	MOVWF	EE_DAT				; |  "
	CALL	DM_WR				; / write 04 byte now
					; read eeprom (the actual data bytes)
EEP_RD1	MOVLW	d'01'				; \ set 1st address to read
	MOVWF	EE_ADD				; |  "
	CALL	DM_RD				; | read 01 byte now
	MOVWF	CFG_RA				; |  save
	CALL	DM_RD				; | read 02 byte now
	MOVWF	CFG_RB				; |  save
	CALL	DM_RD				; | read 03 byte now
	MOVWF	CFG_LL				; |  save
	CALL	DM_RD				; | read 04 byte now
	MOVWF	CFG_SC				; /  save

;************
; programming mode ?  (could possibly be simplified)
;  (PM_CNT1 is just a temporary variable)
					; get inputs
START4	MOVF	PORTC,W				; \ read Port-C
	ANDLW	b'00000111'			; |  mask unused bits
	BCF	STATUS,C			; | clear carry for rotate
	MOVWF	PM_CNT1				; | temp save for shift
	RLF	PM_CNT1,F			; |  shift left
	RLF	PM_CNT1,F			; |  shift left
	RLF	PM_CNT1,F			; |  shift left
	MOVF	PORTA,W				; | read Port-A
	ANDLW	b'00000111'			; |  mask unused bits
	IORWF	PM_CNT1,W			; | W = true input states
	MOVWF	PM_CNT1				; /  save
 					; already done ?
	BTFSC	FLAGS1,1			; already detected ?
	GOTO	START5				;  yes, wait for release
					; process
	MOVF	PM_CNT1,W			; read input state
	ANDLW	b'00111101'			; mask unwanted inputs
	SUBLW	b'00011100'			; compare
	BTFSS	STATUS,Z			;  z = yes
	GOTO	START6				;  not programming mode, continue
	BSF	FLAGS1,1			; flag programming mode = yes
					; 2 short LED flahses
START5	BSF	PORTA,LED_OP			; LED on
	CALL	DEL60				; 60 mS delay
	BCF	PORTA,LED_OP			; LED off
	CALL	DEL60				; 60 mS delay
	CALL	DEL60				; 60 mS delay
	BSF	PORTA,LED_OP			; LED on
	CALL	DEL60				; 60 mS delay
	BCF	PORTA,LED_OP			; LED off
					; released yet ?
	MOVF	PM_CNT1,W			; read input state
	ANDLW	b'00011100'			; are any of the inputs still active ?
	BTFSS	STATUS,Z			;  z = no
	GOTO	START4				; no, wait
					; clean up
START6	CLRF	SWACT				;  clear
	CLRF	SWDBL				;   clear
	CLRF	SWLONG				;    clear

;************
; complete startup

START9	BSF	FLAGS2,2			; enable LED pulse
	BSF	FLAGS2,3			; backlights on
	MOVLW	d'5'				; \
	MOVWF	PM_CNT1				; | preset counters
	MOVWF	PM_CNT4				; /

;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;		M A I N   P R O G R A M   L O O P
;******************************************************************************

CORE:	BTFSS	FLAGS1,0			; 10mS Core-Loop flag ?
	GOTO	CORE				;  no, wait
	BCF	FLAGS1,0			;  yes, reset flag and run core-loop
	CLRWDT					; reset watchdog timer

;************
; Programming

; Enter by activating inputs 3,4,5 and applying power.
;  Release when you see the 2 quick flashes from the on-board LED.
;  The backlight output flashes when in programming mode.
; Input-3 = function sequence
; Input-5 = count (value)
; Input-3 = long press to save
; Input-1 = press to reset function and count to zero
; Input-1 = long press to quit programming

PM_001	BTFSS	FLAGS1,1			; programming mode ?
	GOTO	CORE10				;  no, skip
					; cycle rate (= 200mS)
	DECF	PM_CNT1,F			; cycle counter
	BTFSS	STATUS,Z			;
	GOTO	PM_099				;
	MOVLW	d'20'				; x10mS (rate)
	MOVWF	PM_CNT1				;
					; input press programming
					; reset counts
PM_002	BTFSS	SWACT,0				; reset activated ?
	GOTO	PM_003				;  no, skip
	CLRF	PM_CNT2				; clear count #1 "presses"
	CLRF	PM_CNT3				; clear count #2 "presses"
	CLRF	SWDBL				; clear
	CLRF	SWLONG				;  clear
	CLRF	SWACT				;   clear
					; #1 = select function
PM_003	BTFSS	SWACT,2				; input-2 activated ?
	GOTO	PM_004				;  no, skip
	INCF	PM_CNT2,F			; count #1 "presses"
	BCF	SWACT,2				;  clear
					; #2 = enter count/value
PM_004	BTFSS	SWACT,4				; input-4 activated ?
	GOTO	PM_005				;  no, skip
	INCF	PM_CNT3,F			; count #2 "presses"
	BCF	SWACT,4				;  clear
					; #1 (long press to save)
PM_005	BTFSS	SWLONG,2			; input-2 long ?
	GOTO	PM_010				;  no, skip
	MOVF	PM_CNT2,W			; \
	SUBLW	d'01'				; | function 1 ? (Relay-A timeout)
	BTFSS	STATUS,Z			; |
	GOTO	PM_006				; /
	CLRW					; \
	ADDWF	PM_CNT3,W			; | data to save
	ADDWF	PM_CNT3,W			; |  x2
	ADDWF	PM_CNT3,W			; |  x3
	ADDWF	PM_CNT3,W			; |  x4
	ADDWF	PM_CNT3,W			; |  x5
	MOVWF	CFG_RA				; |  make it active
	MOVWF	EE_DAT				; |  data to write
	MOVLW	d'01'				; | eeprom address to write
	MOVWF	EE_ADD				; |  "
	CALL	DM_WR				; / write byte now
	CLRF	PM_CNT2				; clear count #1 "presses"
	CLRF	PM_CNT3				; clear count #2 "presses"
	BCF	SWLONG,2			;  clear
	GOTO	PM_010				; done
PM_006	MOVF	PM_CNT2,W			; \
	SUBLW	d'02'				; | function 2 ? (Relay-B timeout)
	BTFSS	STATUS,Z			; |
	GOTO	PM_007				; /
	CLRW					; \
	ADDWF	PM_CNT3,W			; | data to save
	ADDWF	PM_CNT3,W			; |  x2
	ADDWF	PM_CNT3,W			; |  x3
	ADDWF	PM_CNT3,W			; |  x4
	ADDWF	PM_CNT3,W			; |  x5
	MOVWF	CFG_RB				; |  make it active
	MOVWF	EE_DAT				; |  data to write
	MOVLW	d'02'				; | eeprom address to write
	MOVWF	EE_ADD				; |  "
	CALL	DM_WR				; / write byte now
	CLRF	PM_CNT2				; clear count #1 "presses"
	CLRF	PM_CNT3				; clear count #2 "presses"
	BCF	SWLONG,2			;  clear
	GOTO	PM_010				; done
PM_007	MOVF	PM_CNT2,W			; \
	SUBLW	d'03'				; | function 3 ? (Light level)
	BTFSS	STATUS,Z			; |
	GOTO	PM_008				; /
	MOVF	PM_CNT3,W			; data count
	BTFSC	STATUS,Z			;  ADCRES1 or disable ?
	GOTO	PM_007A				;  read ADCRES1
	MOVLW	h'FF'				;  disable
	GOTO	PM_007B				;   "
PM_007A	MOVF	ADCRES1,W			; \ data to save
PM_007B	MOVWF	CFG_LL				; |  make it active
	MOVWF	EE_DAT				; |  data to write
	MOVLW	d'03'				; | eeprom address to write
	MOVWF	EE_ADD				; |  "
	CALL	DM_WR				; / write byte now
	CLRF	PM_CNT2				; clear count #1 "presses"
	CLRF	PM_CNT3				; clear count #2 "presses"
	BCF	SWLONG,2			;  clear
	GOTO	PM_010				; done
PM_008	MOVF	PM_CNT2,W			; \
	SUBLW	d'04'				; | function 4 ? (Switch config)
	BTFSS	STATUS,Z			; |
	GOTO	PM_009				; /
	MOVF	PM_CNT3,W			; \ data to save
	MOVWF	CFG_SC				; |  make it active
	MOVWF	EE_DAT				; |  data to write
	MOVLW	d'04'				; | eeprom address to write
	MOVWF	EE_ADD				; |  "
	CALL	DM_WR				; / write byte now
	CLRF	PM_CNT2				; clear count #1 "presses"
	CLRF	PM_CNT3				; clear count #2 "presses"
	BCF	SWLONG,2			;  clear
	GOTO	PM_010				; done
PM_009


; NEW FUNCTION HERE

					; flash the backlight
PM_010	DECF	PM_CNT4,F			; cycle counter
	BTFSS	STATUS,Z			;
	GOTO	PM_099				;
	MOVLW	d'03'				; 600mS
	MOVWF	PM_CNT4				;
	MOVF	FLAGS2,W			; \ read port
	XORLW	1 << 3				; |  invert
	MOVWF	FLAGS2				; / write port
					; done
PM_099	GOTO	CORE90				; loop (skip relay logic)

;******************************************************************************
; input-1 resets both relays
;	    cancels both relay timers
; input-2 toggles relay-A
;	    cancels relay_A timers
; input-3 activated turns on relay-A with a 5 minute timeout
; input-4 toggles relay-B
;	    cancels relay_A timers
; input-5 activated turns on relay-B with a 5 minute timeout

;************
; Relay Logic
					; A toggle
CORE10	BTFSS	SWACT,2				; input-2 activated ?
	GOTO	CORE11				;  no, skip
;	BTFSC	CFG_SC,0			; auto-light relay-A ?
;	GOTO	CORE21				;  no, skip with SWACT clear
	MOVF	PORTC,W				; \ read port
	XORLW	1 << RELAY_A			; |  invert
	MOVWF	PORTC				; / write port
	BTFSS	PORTC,RELAY_A			; is relay still on
	BCF	FLAGS2,7			;  no, stop relay_A timer
CORE11	BCF	SWACT,2				; clear input
					; A on with timeout and light sensor
	BTFSS	SWACT,3				; input-3 activated ?
	GOTO	CORE12				;  no, skip
	BTFSC	CFG_SC,0			; auto-light relay-A ?
	GOTO	CORE11C				;  yes, skip light level qualifier
	INCF	CFG_LL,W			; light level
	BTFSC	STATUS,W			; is it FF (disabled)
	GOTO	CORE11C				;  yes, light on now
	MOVF	CFG_LL,W			; \ read light level
	SUBWF	ADCRES1,W			; |  compare
	BTFSS	STATUS,C			; | is it dark enough ?
	GOTO	CORE12				; /  no, skip
CORE11C	BSF	PORTC,RELAY_A			; \ relay on
	MOVF	CFG_RA,W			; | \ preset timer count  (x1-min)
	MOVWF	CNT_RA				; | |  "
	BSF	FLAGS2,6			; | / start timer
CORE12	BCF	SWACT,3				; / clear input
					; B toggle
CORE20	BTFSS	SWACT,4				; input-2 activated ?
	GOTO	CORE21				;  no, skip
	MOVF	PORTC,W				; \ read port
	XORLW	1 << RELAY_B			; |  invert
	MOVWF	PORTC				; / write port
	BTFSS	PORTC,RELAY_B			; is relay still on
	BCF	FLAGS2,7			;  no, stop relay_B timer
CORE21	BCF	SWACT,4				; clear input
					; B on with timeout
	BTFSS	SWACT,5				; input-3 activated ?
	GOTO	CORE22				;  no, skip
	INCF	CFG_LL,W			; light level
	BTFSC	STATUS,W			; is it FF (disabled)
	GOTO	CORE21C				;  yes, light on now
	MOVF	CFG_LL,W			; \ read light level
	SUBWF	ADCRES1,W			; |  compare
	BTFSS	STATUS,C			; | is it dark enough ?
	GOTO	CORE22				; /  no, skip
CORE21C	BSF	PORTC,RELAY_B			; \ relay on
	MOVF	CFG_RB,W			; | \ preset timer count  (x1-min)
	MOVWF	CNT_RB				; | |  "
	BSF	FLAGS2,7			; | / start timer
CORE22	BCF	SWACT,5				; / clear input
					; A auto-light
CORE30	BTFSS	CFG_SC,0			; auto-light relay-A ?
	GOTO	CORE40				;  no, skip
	BTFSC	FLAGS1,4			; already done - on ?
	GOTO	CORE31				;  yes, check for off
	MOVF	CFG_LL,W			; \ read light level
	ADDLW	d'08'				; | add some hysteresis
	SUBWF	ADCRES1,W			; |  compare
	BTFSS	STATUS,C			; | is it dark enough ?
	GOTO	CORE40				; /  no, skip
	BSF	PORTC,RELAY_A			; relay on
	BSF	FLAGS1,4			; flag as done
	GOTO	CORE40				; done
CORE31	MOVF	CFG_LL,W			; \ read light level
	SUBWF	ADCRES1,W			; |  compare
	BTFSC	STATUS,C			; | is it light enough ?
	GOTO	CORE40				; /  no, skip
	BCF	PORTC,RELAY_A			; relay off
	BCF	FLAGS1,4			; clear done flag
	GOTO	CORE40				; done
					; reset
CORE40	BTFSS	SWACT,0				; reset activated ?
	GOTO	CORE41				;  no, skip
	BTFSS	CFG_SC,0			; leave relay-A if autolight
	BCF	PORTC,RELAY_A			; relay off
	BCF	PORTC,RELAY_B			; relay off
	BCF	FLAGS2,6			; stop relay_A timer
	BCF	FLAGS2,7			; stop relay_B timer
CORE41	BCF	SWACT,0				; clear reset


;************
; LOOP CORE
					; clean up (for debugging)
CORE90	BTFSS	SWLONG,0			; long reset ?
	GOTO	CORE99				;  no, skip
	BCF	FLAGS1,4			; clear auto-light done flag
	BCF	FLAGS1,1			; cancel programming mode
	BSF	FLAGS2,3			;  ensure backlight is on
	CLRF	SWDBL				; clear
	CLRF	SWLONG				;  clear
	CLRF	SWACT				;   clear
					; loop
CORE99	CLRF	SWACT				; clean up
	CLRF	SWDBL				; clean up
	CLRF	SWLONG				; clean up
	GOTO	CORE				; loop

;************************END OF MAIN LOOP****************************

;;************
;; Relay_A follow

;CORE30	BTFSS	SWACT,3				; input-3 activated ?
;	GOTO	CORE31				;  no, skip
;	BSF	PORTC,RELAY_A			; relay on
;	GOTO	CORE32
;CORE31	BCF	PORTC,RELAY_A			; relay off
;CORE32	BCF	SWACT,3				; clear input

;;************
;; Relay_B toggle (QUALIFIED)
;
;CORE20	BTFSS	INPUT1,5			; input-5 active ?
;	GOTO	CORE21				;  no, skip
;	BTFSS	SWACT,4				; input-4 activated ?
;	GOTO	CORE21				;  no, skip
;	MOVF	PORTC,W				; read port
;	XORLW	1 << RELAY_B			;  invert
;	MOVWF	PORTC				; write port
;CORE21	BCF	SWACT,4				; clear input
;	BCF	SWDBL,4				; clear input
;	BCF	SWLONG,4			; clear input
;	BCF	SWACT,5				; clear input
;	BCF	SWDBL,5				; clear input
;	BCF	SWLONG,5			; clear input



;******************************************************************************
;		C O R E   S U B R O U T I N E S
;******************************************************************************






;******************************************************************************

;	MOVF	PORTA,W				; 1 read misc-op byte
;	XORLW	1 << LED_OP			; 1 invert LED power output
;	MOVWF	PORTA				; 1 write misc-op byte

;	MOVF	FLAGS2,W			; read
;	XORLW	1 << 3				; invert DIAGNOSTIC FLAG
;	MOVWF	FLAGS2				; write


;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;		G E N E R A L   S U B R O U T I N E S
;******************************************************************************

; add more here

;******************************************************************************
;******************************************************************************
; PIC internal data-eeprom read/write subroutines
;******************************************************************************

;************
; Read from PIC internal data EEPROM

; The address to read is passed in EE_ADD which is incremented after read.
; The data read is returned in W

; read example
	;	MOVLW	???		; address to read
	;	MOVWF	EE_ADD		; save address
	;	CALL	DM_RD		; read first address
	;	MOVWF	???		; save data from eeprom
	;	CALL	DM_RD		; read next address
	;	MOVWF	???		; save data from eeprom

					; read 1 byte (addressed by W)
DM_RD	MOVF	EE_ADD,W			; get address to read
	CALL	INT_OFF				; disable interrupts
	BSF	STATUS,RP0			; select bank 1
	MOVWF	EEADR				; set address to read
	BSF	EECON1,RD			;  read
	MOVF	EEDAT,W				; read data byte into "W"
	BCF	STATUS,RP0			; select bank 0
	BSF	INTCON,GIE			; global interrupt enable
	INCF	EE_ADD,F			; point to next address
	RETURN					; finished

;************
; Write 1-byte to PIC internal data EEPROM

; The byte in "EE_DAT" is written to the address in "EE_ADD".
; This routine waits for the WR flag to clear before initiating another
;  write cycle. This only slows consecutive writes.
; Interrupts are not disabled until after WR is clear to minimse IRQ down time.

DM_WR	CALL	INT_OFF				; disable interrupts
	BSF	STATUS,RP0			; \ select bank 1
DM_WR1	CLRWDT					; | clear watchdog
	BTFSC	EECON1,WR			; | wait for last write to complete
	GOTO	DM_WR1				; |  loop
	BCF	STATUS,RP0			; | select bank-0
	BCF	PIR1,EEIF			; / clear interrupt flag
	BSF	STATUS,RP0			; select bank 1
	MOVF	EE_DAT,W			; get data byte into W
	MOVWF	EEDAT				; load data to write
	MOVF	EE_ADD,W			; get address
	MOVWF	EEADR				; save address to write
	BSF	EECON1,WREN			; write enable
	MOVLW	55H				;   write access sequence
	MOVWF	EECON2				;    "
	MOVLW	0AAH				;    "
	MOVWF	EECON2				;    "
	BSF	EECON1,WR			;   begin write
	BCF	EECON1,WREN			; write disable (safety)
	BCF	STATUS,RP0			; select bank 0
	BSF	INTCON,GIE			; global interrupt enable
	INCF	EE_ADD,F			; point to next address
	RETURN					; finished

;************
; global interrupt disable

INT_OFF	BCF	INTCON,GIE			; \
	BTFSC	INTCON,GIE			; | global interrupt disable
	GOTO	INT_OFF				; /
	RETURN					; done

;************************************************************************
; Short Delays
;************************************************************************

; bit delay (depends on clock speed)

DELAYS:

BIT_DLY	MOVLW	d'10'				; delay time
	MOVWF	GPTIM3				; counter

BDEL1	CLRWDT					; reset watchdog timer
	DECFSZ	GPTIM3,F			; test counter
	GOTO	BDEL1				; loop
	RETURN					; finished

; 60mS 	(+ or - 10mS) (RTI based)

DEL60	MOVLW	d'06'				; delay time
	MOVWF	GPTIM1				; counter

DEL601	CLRWDT					; reset watchdog timer
	MOVF	GPTIM1,W			; test counter
	BTFSS	STATUS,Z			;
	GOTO	DEL601				; loop
	RETURN					; finished

; 2 SEC (+ or - 10mS) (RTI based)

DEL25S	MOVLW	d'25'				; delay time (0.25 seconds)
	MOVWF	GPTIM1				; counter
	GOTO	DEL2S1				; continue
DEL2S	MOVLW	d'200'				; delay time (2 seconds)
	MOVWF	GPTIM1				; counter

DEL2S1	CLRWDT					; reset watchdog timer
	MOVF	GPTIM1,F			; test counter
	BTFSS	STATUS,Z			;
	GOTO	DEL2S1				; loop
	RETURN					; finished

;******************************************************************************
; Data I/O ROUTINES
;******************************************************************************

; Send data in W out via external I/O.
; 8 bits, short pulse = low, long pulse = high, MSB first.
; Must only be call from withn an interrupt (must not be interrupted).

DAT_TX:	MOVWF	TMP2				; save data to working reg.
	MOVLW	d'08'				; bit counter
	MOVWF	TMP1				;  "
	BSF	STATUS,RP0			; \ select bank 1
	BCF	TRISA,EXT_IO			; | make pin = output
	BCF	STATUS,RP0			; / select bank 0

	BCF	PORTA,EXT_IO			; make output low
	NOP					;  1 bit delay
					; loop
DATTX1	BSF	PORTA,EXT_IO			;  \ make output high
	RLF	TMP2,F				;  | get data bit
	BTFSS	STATUS,C			;  | test
	BCF	PORTA,EXT_IO			;  | make data low
	NOP					;  |  3 bit delay
	NOP					;  |   "
	NOP					;  |   "
	BCF	PORTA,EXT_IO			;  | make data low
	NOP					;  |  1 bit delay
	DECFSZ	TMP1,F				;  | bit count -1
	GOTO	DATTX1				;  / loop
					; done
	BSF	STATUS,RP0			; \ select bank 1
	BSF	TRISA, EXT_IO			; | make pin = input
	BCF	STATUS,RP0			; / select bank 0
	RETURN					; done, return

;******************************************************************************
; OSC Calibration
;******************************************************************************

;	ORG	h'03FF'	        	; OSC calibration address
;	RETLW	h'54'			; config data byte

;******************************************************************************
;******************************************************************************

	END
